;; 
;;
PRO ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR__DEFINE

    ; define the class structure 
    struct = { ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR, duration:0.0, scale_filter:obj_new() }

END


;; 
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::INIT, DURATION=DURATION, LOWER_SCALE_BOUNDARY=LOWER_SCALE_BOUNDARY, UPPER_SCALE_BOUNDARY=UPPER_SCALE_BOUNDARY

    ; try to initialize field values
    catch, e
    if (e ne 0) then begin

        message, !error_state.msg, /informational
        return, 0

    endif

        ; try to initialize field values
        self->set_duration, duration

    catch, /cancel

    ; initialize the scale filter
    self.scale_filter = obj_new('scale_filter', multiplex_dimension=2, lower_boundary=lower_scale_boundary, upper_boundary=upper_scale_boundary)
    if (~obj_valid(self.scale_filter)) then begin
        return, 0
    endif

    return, 1

END


;;
;;
PRO ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::CLEANUP

    obj_destroy, self.scale_filter

END


;; TODO check if given value is a valid number
;;
PRO ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::SET_DURATION, DURATION

    ; reset to -1.0, if no value given
    if (n_elements(duration) eq 0) then begin
        self.duration = -1.0
        return
    endif

    self.duration = duration

END


;;
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::GET_DURATION

    return, self.duration

END



;;
;;
PRO ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::SET_LOWER_SCALE_BOUNDARY, BOUNDARY

    (self.scale_filter)->set_lower_boundary, boundary

END


;;
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::GET_LOWER_SCALE_BOUNDARY

    return, (self.scale_filter)->get_lower_boundary()

END


;;
;;
PRO ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::SET_UPPER_SCALE_BOUNDARY, BOUNDARY

    (self.scale_filter)->set_upper_boundary, boundary

END


;;
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::GET_UPPER_SCALE_BOUNDARY

    return, (self.scale_filter)->get_upper_boundary()

END


;; 
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::COMPARE, RESPIRATORY_SIGNALS, OFFSETS

    n_signals = n_elements(respiratory_signals)
    comparison_matrix = rebin([0.0], n_signals, n_signals)
    comparison_matrix[indgen(n_signals) * (n_signals+1)] = 1.0
    return, comparison_matrix

END


;;
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::COMP, SIGNAL0, SIGNAL1

    return, (self->compare([signal0, signal1]))[0,1]

END


;;
;; @param RESPIRATORY_SIGNALS (IN) An array of RESPIRATORYSIGNAL objects to resample.
;; @param OFFSETS (?IN) An array of offsets into each given RESPIRATORYSIGNAL, given in seconds. 
;;                      All samples before the respective offset time are ignored during resampling. 
;;                      If no or not enough offsets for each RESPIRATORYSIGNAL are present, the missing ones are assumed 0.
;; @key TRUNCATE (/IN) 
;; @key SCALE (/IN) 
;; @key SAMPLING_TIME (IN/OUT) The desired sampling time of the resampled signals. If this keyword is undefined, the 
;;                             sampling time is calculated automatically. The calculated value can be reported back through
;;                             this keyword, if present.
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_COMPARATOR::RESAMPLE, RESPIRATORY_SIGNALS, OFFSETS, TRUNCATE=TRUNCATE, SCALE=SCALE, SAMPLING_TIME=SAMPLING_TIME

    n_signals = n_elements(respiratory_signals)

    ; calculate sampling time, if none was specified
    if (n_elements(sampling_time) eq 0) then begin
    
        ; check all given respiratory signals and determine the maximum sampling time
        max_sampling_time = 0.0
        for i = 0, n_signals-1 do begin
            if (~obj_isa(respiratory_signals[i], 'respiratorysignal')) then begin
                message, 'All elements of the given array must be objects of the type RESPIRATORYSIGNAL!'
            endif
            max_sampling_time >= (respiratory_signals[i])->getsamplingtime()
        endfor
        sampling_time = max_sampling_time

    endif

    ; convert the array of offsets from seconds to numbers of samples
    ; assume offsets not specified or specified incompletely to be 0
    n_offsets = n_elements(offsets)
    if (n_offsets eq 0) then begin
        sample_offsets = intarr(n_signals)
    endif else if (n_offsets lt n_signals) then begin
        sample_offsets = [round(offsets/sampling_time), intarr(n_signals-n_offsets)] 
    endif else begin
        sample_offsets = round(offsets[0:n_signals-1]/sampling_time)
    endelse

    ; determine the sizes of the resampled signals starting from the their respective offset
    resampled_sizes = lindgen(n_signals)
    for i = 0, n_signals-1 do begin
        resampled_sizes[i] = (respiratory_signals[i])->getsize() * (respiratory_signals[i])->getSamplingTime()
        resampled_sizes[i] /= sampling_time
        resampled_sizes[i] -= sample_offsets[i]
    endfor

    ; create an array for all resampled signals
    ; the size is determined according to the desired duration of the comparison, if setup
    ; it is also determined by the truncate keyword, stating if data need to be truncated, or can be filled up with NaN values
    if (keyword_set(truncate)) then begin
        min_sample = min(sample_offsets)
        if (min_sample lt 0) then begin
            sample_offsets -= min_sample
            resampled_sizes += min_sample
        endif 
        if (self.duration gt 0.0) then begin
            n_samples = min(resampled_sizes) < floor(self.duration / sampling_time)
        endif else begin
            n_samples = min(resampled_sizes)
        endelse
        signal_matrix = rebin([!values.f_nan], n_signals, n_samples)
        for i = 0, n_signals-1 do begin
            signal_matrix[i,0:n_samples-1] = ((respiratory_signals[i])->getresampledsignal(sampling_time))[sample_offsets[i]:sample_offsets[i]+n_samples-1]
        endfor
    endif else begin
        if (self.duration gt 0.0) then begin
            n_samples = floor(self.duration / sampling_time)
        endif else begin
            n_samples = max(resampled_sizes)
        endelse
        signal_matrix = rebin([!values.f_nan], n_signals, n_samples)
        for i = 0, n_signals-1 do begin
            current_offset = sample_offsets[i]
            current_size = resampled_sizes[i] < n_samples
            if (current_offset ge 0) then begin
                signal_matrix[i,0:current_size-1] = ((respiratory_signals[i])->getresampledsignal(sampling_time))[current_offset:current_offset+current_size-1]
            endif else begin
                signal_matrix[i,-current_offset:current_size-1] = ((respiratory_signals[i])->getresampledsignal(sampling_time))[0:current_offset+current_size-1]
            endelse
        endfor
    endelse

    if (keyword_set(scale)) then begin
        signal_matrix = (self.scale_filter)->multiplex_apply(signal_matrix)
    endif

    return, signal_matrix

END
