;; Calculates the average of the locations of a group of markers.
;; For each sample frame the center of all marker locations is determined. 
;; This results in a more compact and stable representation of the marker movements,
;; while discarding specific information for single markers. Note, that some marker 
;; movements can eliminate each other, and are therefore not seen in the average.
;; @param MARKERS (IN) A three-dimensional array of markers, as returned by the GET_ART_MARKER_ARRAY method.
;;                     The first dimension corresponds to the sample frames. 
;;                     The second dimension corresponds to the slots for different markers. 
;;                     The third dimension corresponds to the 3D spatial coordinates of the markers.
;; @return A two-dimensional array of marker movements.
;;         The first dimension corresponds to the sample frames.
;;         The second dimension corresponds to the 2D spatial coordinates of the average of all markers of the frame.
;;
FUNCTION AVERAGE_MARKER_DATA, MARKERS

    ; check marker array
    if (size(markers, /n_dimensions) ne 3) then begin
        message, 'The array of markers must have exactly three dimensions! (Sample frames, markers, 3D-coordinates)'
    endif
    marker_dimensions = size(markers, /dimensions)
    if (marker_dimensions[2] ne 3) then begin
        message, 'The array of markers must contain 3d vectors!'
    endif

    ; for each frame and dimension, return the average over all markers
    return, total(markers, 2) / marker_dimensions[1]

END


;; Calculates a primary components analysis (PCA) of the locations of a group of markers.
;; The PCA determins the variance of the marker movements over all sample frames, for each dimension
;; of each marker. Then it sets up a new coordinate system with an appropriate number of dimensions, 
;; where the dimensions are sorted according to the maximum variance of the marker movements. Then, the 
;; original data are transformed into that new coordinate system. This results in the most variant 
;; (and probably most important) information to apear in the first component of the PCA output.
;; The PCA is computed over all three dimensions of the marker locations. Depending on the keyword 
;; PER_MARKER, it is computed for each marker seperately, or for all markers together. 
;; @param MARKERS (IN) A three-dimensional array of markers, as returned by the GET_ART_MARKER_ARRAY method.
;;                     The first dimension corresponds to the sample frames. 
;;                     The second dimension corresponds to the slots for different markers. 
;;                     The third dimension corresponds to the 3D spatial coordinates of the markers.
;; @arg PER_MARKER (?IN) A boolean keyword which specifies, if the PCA is computed over all data or per marker.
;; @return A multidimensional array containing the components resulting from PCA.
;;         The first dimension always corresponds to the sample frames. 
;;         If PER_MARKER is not set, the second dimension corresponds the components resulting from a PCA
;;         of all dimensions of all markers put together. 
;;         If PER_MARKER is set, the second dimension corresponds to the single markers. The additional third 
;;         dimension then contains the components of a PCA over the three dimensions of the respective marker.
;;
FUNCTION CALCULATE_ART_PCA, MARKERS, PER_MARKER=PER_MARKER, _REF_EXTRA=extra

    ; check marker array
    if (size(markers, /n_dimensions) ne 3) then begin
        message, 'The array of markers must have exactly three dimensions! (Sample frames, markers, 3D-coordinates)'
    endif
    marker_dimensions = size(markers, /dimensions)
    if (marker_dimensions[2] ne 3) then begin
        message, 'The array of markers must contain 3d vectors!'
    endif

    ; calculate per marker pcas or combined pca?
    if (keyword_set(per_marker)) then begin

        n_samples = marker_dimensions[0]
        n_markers = marker_dimensions[1]
        pcas = fltarr(n_markers, n_samples, 3)
        
        ;iterate over each marker
        for i = 0, n_markers-1 do begin
            pcas[i,*,*] = pcomp(transpose(markers[*,i,*], [2,0,1]))
        endfor

        return, pcas

    endif else begin
    
        ; calculate combined pca of all markers
        n_samples = marker_dimensions[0]
        n_dimensions = marker_dimensions[1] * 3
        combined_markers = transpose(reform(markers, n_samples, n_dimensions))
        
        pca = pcomp(combined_markers, /double, _EXTRA=extra)
        return, pca

    endelse
    
END


;;
;;
FUNCTION PROJECT_ART_TO_VECTOR, MARKERS, VECTOR, NORMALIZE=NORMALIZE

    ; check marker array
    if (size(markers, /n_dimensions) ne 3) then begin
        message, 'The array of markers must have exactly three dimensions! (Sample frames, markers, 3D-coordinates)'
    endif
    marker_dimensions = size(markers, /dimensions)
    if (marker_dimensions[2] ne 3) then begin
        message, 'The array of markers must contain 3d vectors!'
    endif
    n_dimensions = marker_dimensions[1] * 3
    n_samples = marker_dimensions[0]

    ; check projection vector
    if (n_elements(vector) ne n_dimensions) then begin
        message, 'The projection vector must contain one component for each dimension of all markers!'
    endif

    ; reform and process target vector of projection
    vect = reform(vector, 1, n_dimensions)
    vect_square = total(vect^2)
    if (keyword_set(normalize)) then begin
        vect /= sqrt(vect_square)
        vect_square = 1.0
    endif

    ; project the art markers to the target vector
    combined_markers = reform(markers, n_samples, n_dimensions)
    base_vectors = rebin(vect, n_samples, n_dimensions)
    projected_markers = (combined_markers * base_vectors) / vect_square
    
    return, total(projected_markers, 2)

END


;; Gets the relative movement of a marker over a certain period of time.
;; For a given marker, the movement vector from one sample frame to another is calculated.
;; This does not account for any movements inbetween, but rather assumes a direct movement
;; between the two locations. (This method can be used for calibration, if a well defined 
;; movement of markers was performed.)
;; @param MARKERS (IN) A three-dimensional array of markers, as returned by the GET_ART_MARKER_ARRAY method.
;;                     The first dimension corresponds to the sample frames. 
;;                     The second dimension corresponds to the slots for different markers. 
;;                     The third dimension corresponds to the 3D spatial coordinates of the markers.
;; @param SLOT (IN) The marker whose movement to get. This is specified as a slot number within the given MARKERS.
;; @param START_FRAME (IN) The offset of the sample frame where the desired movement begins.
;; @param END_FRAME (IN) The offset of the sample frame where the desired movement ends.
;; @return A 3D movement vector describing the movement of the given marker over the given period.
;;
FUNCTION GET_MARKER_MOVEMENT, MARKERS, SLOT, START_FRAME, END_FRAME

    ; check marker array
    if (size(markers, /n_dimensions) ne 3) then begin
        message, 'The array of markers must have exactly three dimensions! (Sample frames, markers, 3D-coordinates)'
    endif
    marker_dimensions = size(markers, /dimensions)
    if (marker_dimensions[2] ne 3) then begin
        message, 'The array of markers must contain 3d vectors!'
    endif

    ; determine requested movement
    return, reform(markers[end_frame,slot,*] - markers[start_frame,slot,*])

END


; ************* Output Functions And Procedures ************


;; Saves an array of signal data to a file.
;; In addition to the signal itself, its sampling rate saved.
;; The resulting file uses an IDL specific SAV file format.
;; TODO check parameters 
;; @param SIGNAL_DATA (IN) An array containing the samples of a signal.
;; @param SAMPLING_TIME (IN) The time between two samples of the SIGNAL_DATA.
;; @param FILENAME (IN) The full path to the file to write. It can be absolute or relative.
;;
PRO SAVE_RESPIRATION_CURVE, FILENAME, SIGNAL_DATA, SAMPLING_TIME

    ; create proper data structure, and save it to a file
    resp_signal = obj_new('RespiratorySignal')
    resp_signal->setsignal, signal_data, sampling_time
    resp_signal->save_savfile, filename
    obj_destroy, resp_signal

END


;; 
;;
PRO SAVE_MOVEMENT_OFFSETS, FILENAME, SIGNAL_DATA, SAMPLING_TIME

    ; create proper variables, and save them to a file
    save_resp_Signal = signal_data
    save_resp_SamplingTime = sampling_time
    save_resp_Sensor = "ART"
    save_resp_Comments = ""
    save, filename=filename, save_resp_Signal, save_resp_SamplingTime, save_resp_Sensor, save_resp_Comments

END
