;; Visualizes marker locations as a 3D video.
;; The sampling rate is determined from the timestamps in the given ART data.
;; For each frame, the location of each marker is plotted into an isometric 3D coordinate system.
;; The size of the coordinate system is determined beforehand, so that it fits all
;; marker locations of all frames.
;; TODO Use points or spheres for visualisation, rather then lines
;; @param DATA (IN) An array of ART samples, as returned by the function LOAD_ART_DATA.
;; @extra Additionally arguments of the PLOT procedure are accepted.
;;
PRO PLOT_ART_SAMPLE_VIDEO, DATA, _REF_EXTRA=extra

    ; calculate common coordinate system range for each dimension
    minimum = [!values.f_infinity, !values.f_infinity, !values.f_infinity]
    maximum = [-!values.f_infinity, -!values.f_infinity, -!values.f_infinity]
    for i = 0, n_elements(data)-1 do begin
        if (ptr_valid(data[i].markers)) then begin
            xyz = (*(data[i].markers)).loc;
            xyz = reform(xyz, 3, n_elements(*(data[i].markers)), /overwrite)
            current_max = max(xyz, min=current_min, dimension=2)
            minimum <= current_min
            maximum >= current_max
        endif
    endfor
    range = [[minimum],[maximum]]

    for i = 0, n_elements(data)-1 do begin

        ; adjust timing
        if (n_elements(old_ts) gt 0) then begin
            wait, data[i].ts - old_ts
        endif
        old_ts = data[i].ts

        ; plot data
        if (ptr_valid(data[i].markers)) then begin
            xyz = (*(data[i].markers)).loc
            if ((size(xyz, /dimensions))[1] gt 1) then begin
                plot_3dbox, xyz[0,*], xyz[1,*], xyz[2,*], xrange=range[0,*], yrange=range[1,*], zrange=range[2,*], _EXTRA=extra
            endif
        endif

    endfor

END

;; Visualizes marker locations as a 3D video.
;; For each frame, the location of each marker is plotted into an isometric 3D coordinate system.
;; The size of the coordinate system is determined beforehand, so that it fits all
;; marker locations of all frames.
;; TODO Use points or spheres for visualisation, rather then lines
;; @param MARKERS (IN) An three-dimensional array of markers, as returned by the GET_ART_MARKER_ARRAY method.
;;                     The first dimension corresponds to the sample frames. 
;;                     The second dimension corresponds to independent markers. 
;; @param SAMPLING_TIME (?IN) The sampling time to use between plotting two sample frames. 
;;                            If not given, a default value is used.
;; @extra Additionally arguments of the PLOT procedure are accepted.
;;
PRO PLOT_ART_MARKER_VIDEO, MARKERS, SAMPLING_TIME, _REF_EXTRA=extra

    ; check marker array
    if (size(markers, /n_dimensions) ne 3) then begin
        message, 'The array of markers must have exactly three dimensions! (Sample frames, markers, 3D-coordinates)'
    endif
    marker_dimensions = size(markers, /dimensions)
    if (marker_dimensions[2] ne 3) then begin
        message, 'The array of markers must contain 3d vectors!'
    endif
    if (marker_dimensions[1] le 1) then begin
        message, 'There must be at least two markers in order to plot the video!'
    endif

    ; check sampling_time parameter
    if (n_elements(sampling_time) eq 0) then begin
        sampling_time = 0.05
        message, 'No sampling time given. Assuming default: ' + string(sampling_time), /informational
    endif

    ; calculate common coordinate system range for each dimension
    maximum = max(markers, min=minimum, dimension=1, /nan)
    maximum = max(maximum, dimension=1)
    minimum = min(minimum, dimension=1)
    range = [[minimum],[maximum]]

    ; iterate over frames
    for i = 0, (marker_dimensions[0] - 1) do begin

        ; adjust timing
        wait, sampling_time

        ; plot data
        plot_3dbox, markers[i,*,0], markers[i,*,1], markers[i,*,2], xrange=range[0,*], yrange=range[1,*], zrange=range[2,*], _EXTRA=extra

    endfor

END


;; Plots marker locations over time in a common coordinate system.
;; Independent curves for each marker and each dimension are plotted. The size of the coordinate system
;; is chosen to fit all data of all markers.
;; @param MARKERS (IN) An three-dimensional array of markers, as returned by the GET_ART_MARKER_ARRAY method.
;;                     The first dimension corresponds to the sample frames. 
;;                     The second dimension corresponds to independent markers. 
;; @extra Additionally arguments of the PLOT procedure are accepted.
;;
PRO PLOT_ART_MARKERS, MARKERS, _REF_EXTRA=extra

    ; calculate common coordinate system range
    maximum = max(markers, min=minimum, /nan)
    range = [minimum, maximum]

    ; plot each dimension of each marker as a single curve
    for i = 0, (size(markers, /dimensions))[1]-1 do begin
        plot, markers[*,i,0], yrange=range, /noerase, _EXTRA=extra
        plot, markers[*,i,1], yrange=range, /noerase, _EXTRA=extra
        plot, markers[*,i,2], yrange=range, /noerase, _EXTRA=extra
    end
    
END
